/* --*-C-*-- */
/* $Id: options.c.pl,v 1.12 2002/06/19 19:02:40 itz Exp $ */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <getopt.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

#include "options.h"

static struct option gim_options[] = {
{ "age_limit", required_argument, 0, 'a' },
{ "buttons", required_argument, 0, 'b' },
{ "device", required_argument, 0, 'd' },
{ "file", required_argument, 0, 'f' },
{ "foreground", no_argument, 0, 'F' },
{ "help", no_argument, 0, 'h' },
{ "interval", required_argument, 0, 'i' },
{ "ignore", required_argument, 0, 'I' },
{ "protocol", required_argument, 0, 'p' },
{ "pointer", no_argument, 0, 'P' },
{ "repeater", required_argument, 0, 'R' },
{ "scale_down", required_argument, 0, 's' },
{ "scale_up", required_argument, 0, 'S' },
{ "threshold", required_argument, 0, 't' },
{ "verbose", no_argument, 0, 'v' },
{ "version", no_argument, 0, 'V' },
{ "wchars", required_argument, 0, 'w' },

  { 0, 0, 0, 0 }
};

static const char gim_docstrings[] = 
"Distributes gii mouse events to programs on text mode virtual terminals.\n"
"Usage: gim [OPTION]... [CONFIG_FILE]\n\n"
" -a, --age_limit" "=AGE_LIMIT" "  " "Disable pasting selection after AGE_LIMIT seconds"
 "\n" 
" -b, --buttons" "=BUTTONS" "      " "Reinterpret gii button numbers according to BUTTONS"
 "\n" 
" -d, --device" "=DEVICE" "        " "Mouse attached to DEVICE (can be a symlink)"
 "\n" 
" -f, --file" "=FILE" "            " "Listen on Unix address FILE"
 "\n" 
" -F, --foreground" "           " "Do not drop into background; log events on console"
 "\n" 
" -h, --help" "                 " "Print help information successfully"
 "\n" 
" -i, --interval" "=INTERVAL" "    " "Allow INTERVAL milliseconds between double-clicks"
 "\n" 
" -I, --ignore" "=IGNORE" "        " "Ignore terminals listed in IGNORE (e.g. your xdm vt)"
 "\n" 
" -p, --protocol" "=PROTOCOL" "    " "Mouse uses PROTOCOL (see libgii file `inputs.txt')"
 "\n" 
" -P, --pointer" "              " "Draw mouse pointer during selection"
 "\n" 
" -R, --repeater" "=REPEATER" "    " "Use REPEATER as repeater DLL"
 "\n" 
" -s, --scale_down" "=SCALE_DOWN" " " "Scale coordinates in gii events down by SCALE_DOWN"
 "\n" 
" -S, --scale_up" "=SCALE_UP" "    " "Scale coordinates in gii events up by SCALE_UP"
 "\n" 
" -t, --threshold" "=THRESHOLD" "  " "Ignore gii mouse movements smaller than THRESHOLD"
 "\n" 
" -v, --verbose" "              " "Verbosely log all events"
 "\n" 
" -V, --version" "              " "Print version information successfully"
 "\n" 
" -w, --wchars" "=WCHARS" "        " "WCHARS is a list of characters for selection by words"
 "\n" 

  
"\nReport bugs to itz@speakeasy.org.\n"
;

void
print_version_and_copy()
{
  fprintf(stdout, "gim %s protocol %s\n", VERSION, PROTOCOL);
  fprintf(stdout, "Copyright (C) 2002 Ian Zimmerman.\ngim comes with NO WARRANTY, to the extent permitted by law.\nYou may redistribute copies of gim under the terms of the GNU\nGeneral Public License.  For more information about these matters,\nsee the file named GPL.\n");
  exit(0);
}

char* opt_age_limit = 0;
char* opt_buttons = 0;
char* opt_device = 0;
char* opt_file = 0;
int opt_foreground = 0;
int opt_help = 0;
char* opt_interval = 0;
char* opt_ignore = 0;
char* opt_protocol = 0;
int opt_pointer = 0;
char* opt_repeater = 0;
char* opt_scale_down = 0;
char* opt_scale_up = 0;
char* opt_threshold = 0;
int opt_verbose = 0;
int opt_version = 0;
char* opt_wchars = 0;


void
parse_options(int argc, char** argv)
{
  int c;
  int i;
  static char optlist[] = "a:b:d:f:Fhi:I:p:PR:s:S:t:vVw:";
  char* p;

  while ((c = getopt_long(argc, argv, optlist, gim_options, &i)) != -1) {
    switch(c) {
      
	case 'a': opt_age_limit = optarg; break;

	case 'b': opt_buttons = optarg; break;

	case 'd': opt_device = optarg; break;

	case 'f': opt_file = optarg; break;

	case 'F': opt_foreground = 1; break;

	case 'h': opt_help = 1; break;

	case 'i': opt_interval = optarg; break;

	case 'I': opt_ignore = optarg; break;

	case 'p': opt_protocol = optarg; break;

	case 'P': opt_pointer = 1; break;

	case 'R': opt_repeater = optarg; break;

	case 's': opt_scale_down = optarg; break;

	case 'S': opt_scale_up = optarg; break;

	case 't': opt_threshold = optarg; break;

	case 'v': opt_verbose = 1; break;

	case 'V': opt_version = 1; break;

	case 'w': opt_wchars = optarg; break;

	case '?':
	default:
	print_docstrings(1);
    } /*switch*/
  } /*while*/

  if (0 != (p = getenv("gim_age_limit"))) {
    if (opt_age_limit == 0) opt_age_limit = p;
  }
  if (0 != (p = getenv("gim_buttons"))) {
    if (opt_buttons == 0) opt_buttons = p;
  }
  if (0 != (p = getenv("gim_device"))) {
    if (opt_device == 0) opt_device = p;
  }
  if (0 != (p = getenv("gim_file"))) {
    if (opt_file == 0) opt_file = p;
  }
  if (0 != (p = getenv("gim_foreground"))) {
    if (opt_foreground == 0) opt_foreground = atoi(p);
  }
  if (0 != (p = getenv("gim_help"))) {
    if (opt_help == 0) opt_help = atoi(p);
  }
  if (0 != (p = getenv("gim_interval"))) {
    if (opt_interval == 0) opt_interval = p;
  }
  if (0 != (p = getenv("gim_ignore"))) {
    if (opt_ignore == 0) opt_ignore = p;
  }
  if (0 != (p = getenv("gim_protocol"))) {
    if (opt_protocol == 0) opt_protocol = p;
  }
  if (0 != (p = getenv("gim_pointer"))) {
    if (opt_pointer == 0) opt_pointer = atoi(p);
  }
  if (0 != (p = getenv("gim_repeater"))) {
    if (opt_repeater == 0) opt_repeater = p;
  }
  if (0 != (p = getenv("gim_scale_down"))) {
    if (opt_scale_down == 0) opt_scale_down = p;
  }
  if (0 != (p = getenv("gim_scale_up"))) {
    if (opt_scale_up == 0) opt_scale_up = p;
  }
  if (0 != (p = getenv("gim_threshold"))) {
    if (opt_threshold == 0) opt_threshold = p;
  }
  if (0 != (p = getenv("gim_verbose"))) {
    if (opt_verbose == 0) opt_verbose = atoi(p);
  }
  if (0 != (p = getenv("gim_version"))) {
    if (opt_version == 0) opt_version = atoi(p);
  }
  if (0 != (p = getenv("gim_wchars"))) {
    if (opt_wchars == 0) opt_wchars = p;
  }


}

int
print_docstrings (int exit_code)
{
  fprintf(stdout, gim_docstrings);
  exit(exit_code);
  return 0;                     /* not reached */
}

